part of calendar;

class _AllDayAppointmentPainter extends CustomPainter {
  _AllDayAppointmentPainter(this._calendar, this._visibleDates,
      this._visibleAppointment, this._repaintNotifier,
      {this.updateCalendarState})
      : super(repaint: _repaintNotifier);

  final SfCalendar _calendar;
  final List<DateTime> _visibleDates;
  final List<Appointment> _visibleAppointment;
  final ValueNotifier<_AppointmentView> _repaintNotifier;
  final _UpdateCalendarState updateCalendarState;
  double cellWidth, startXPosition, endXPosition, yPosition, cellHeight;
  Paint _rectPainter;
  TextPainter _textPainter;
  BoxPainter _boxPainter;
  final _UpdateCalendarStateDetails _updateCalendarStateDetails =
      _UpdateCalendarStateDetails();

  @override
  void paint(Canvas canvas, Size size) {
    _updateCalendarStateDetails._allDayAppointmentViewCollection = null;
    _updateCalendarStateDetails._currentViewVisibleDates = null;
    updateCalendarState(_updateCalendarStateDetails);
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    _rectPainter = _rectPainter ?? Paint();
    _updateCalendarStateDetails._allDayAppointmentViewCollection =
        _updateCalendarStateDetails._allDayAppointmentViewCollection ??
            <_AppointmentView>[];

    if (_visibleDates != _updateCalendarStateDetails._currentViewVisibleDates) {
      return;
    }

    _rectPainter.isAntiAlias = true;
    cellWidth = size.width / _visibleDates.length;
    cellHeight = size.height;
    startXPosition = 0;
    endXPosition = 0;
    yPosition = 0;
    const double textPadding = 3;

    for (int i = 0;
        i < _updateCalendarStateDetails._allDayAppointmentViewCollection.length;
        i++) {
      final _AppointmentView _appointmentView =
          _updateCalendarStateDetails._allDayAppointmentViewCollection[i];
      if (_appointmentView.canReuse) {
        continue;
      }

      final Appointment _appointment = _appointmentView.appointment;
      int _startIndex = -1;
      int _endIndex = -1;
      final int count = _visibleDates.length;

      int datesCount = 0;
      for (int j = 0; j < count; j++) {
        final DateTime _date = _visibleDates[j];
        if (_date != null &&
            (_date.day == _appointment._actualStartTime.day) &&
            _date.month == _appointment._actualStartTime.month &&
            _date.year == _appointment._actualStartTime.year) {
          _startIndex = datesCount;
          break;
        } else if (_appointment._actualStartTime.isBefore(_date)) {
          _startIndex = datesCount;
          break;
        } else if (_date != null) {
          datesCount++;
        }
      }

      datesCount = 1;
      for (int j = 0; j < count; j++) {
        final DateTime _date = _visibleDates[j];
        if (_date != null &&
            _date.day == _appointment._actualEndTime.day &&
            _date.month == _appointment._actualEndTime.month &&
            _date.year == _appointment._actualEndTime.year) {
          _endIndex = datesCount;
          break;
        } else if (_appointment._actualEndTime.isBefore(_date)) {
          _endIndex = datesCount - 1;
          break;
        } else if (_date != null) {
          datesCount++;
        }
      }
      if (_endIndex == -1) {
        _endIndex = _visibleDates.length;
      }

      const int appointmentHeight = 20;
      startXPosition = _startIndex * cellWidth;
      endXPosition = _endIndex * cellWidth;
      _rectPainter.color = _appointment.color;

      final RRect rect = RRect.fromRectAndRadius(
          Rect.fromLTRB(
              startXPosition,
              (20 * _appointmentView.position).toDouble(),
              endXPosition - textPadding,
              ((appointmentHeight * _appointmentView.position) +
                      appointmentHeight -
                      1)
                  .toDouble()),
          const Radius.circular(
              (appointmentHeight * 0.1) > 2 ? 2 : (appointmentHeight * 0.1)));
      _appointmentView.appointmentRect = rect;
      canvas.drawRRect(rect, _rectPainter);

      final TextSpan span = TextSpan(
        text: _appointment.subject,
        style: _calendar.appointmentTextStyle,
      );
      _textPainter = _textPainter ?? TextPainter();
      _textPainter.text = span;
      _textPainter.textDirection = TextDirection.ltr;
      _textPainter.textAlign = TextAlign.left;
      _textPainter.maxLines = 1;
      _textPainter.textWidthBasis = TextWidthBasis.longestLine;

      _textPainter.layout(
          minWidth: 0,
          maxWidth:
              rect.width - textPadding >= 0 ? rect.width - textPadding : 0);
      if (_textPainter.maxLines == 1 && _textPainter.height > rect.height) {
        continue;
      }

      _textPainter.paint(
          canvas,
          Offset(startXPosition + textPadding,
              rect.top + (rect.height - _textPainter.height) / 2));
      if (_appointment.recurrenceRule != null &&
          _appointment.recurrenceRule.isNotEmpty) {
        double textSize = _calendar.appointmentTextStyle.fontSize;
        if (rect.width < textSize || rect.height < textSize) {
          textSize = rect.width > rect.height ? rect.height : rect.width;
        }

        final TextSpan icon =
            _getRecurrenceIcon(_calendar.appointmentTextStyle.color, textSize);
        _textPainter.text = icon;
        _textPainter.layout(
            minWidth: 0,
            maxWidth:
                rect.width - textPadding >= 0 ? rect.width - textPadding : 0);
        canvas.drawRRect(
            RRect.fromRectAndRadius(
                Rect.fromLTRB(
                    rect.right - textSize, rect.top, rect.right, rect.bottom),
                rect.brRadius),
            _rectPainter);
        _textPainter.paint(
            canvas,
            Offset(rect.right - textSize - 1,
                rect.top + (rect.height - _textPainter.height) / 2));
      }

      if (_repaintNotifier.value != null &&
          _repaintNotifier.value == _appointmentView) {
        Decoration _selectionDecoration = _calendar.selectionDecoration;
        _selectionDecoration ??= BoxDecoration(
          color: Colors.transparent,
          border: Border.all(
              color: const Color.fromARGB(255, 68, 140, 255), width: 2),
          borderRadius: const BorderRadius.all(Radius.circular(1)),
          shape: BoxShape.rectangle,
        );

        Rect rect = _appointmentView.appointmentRect.outerRect;
        rect = Rect.fromLTRB(rect.left, rect.top, rect.right, rect.bottom);
        _boxPainter = _selectionDecoration.createBoxPainter();
        _boxPainter.paint(canvas, Offset(rect.left, rect.top),
            ImageConfiguration(size: rect.size));
      }
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _AllDayAppointmentPainter oldWidget = oldDelegate;
    return oldWidget._visibleDates != _visibleDates ||
        oldWidget._visibleAppointment != _visibleAppointment;
  }
}
